@@ -22,10 +22,36 @@
 //-----------------------------------------------------------------------------
 // Local functions
 
+#ifndef IMAGE_DOS_SIGNATURE
+
+#define IMAGE_DOS_SIGNATURE                 0x5A4D      // MZ
+#define IMAGE_FILE_DLL                      0x2000  // File is a DLL.
+
+typedef struct _IMAGE_DOS_HEADER
+{
+    USHORT e_magic;
+    USHORT dummy[0x1D];
+    DWORD  e_lfanew;
+} IMAGE_DOS_HEADER, *PIMAGE_DOS_HEADER;
+
+typedef struct _IMAGE_FILE_HEADER
+{
+    USHORT  Machine;
+    USHORT  NumberOfSections;
+    DWORD   TimeDateStamp;
+    DWORD   PointerToSymbolTable;
+    DWORD   NumberOfSymbols;
+    USHORT  SizeOfOptionalHeader;
+    USHORT  Characteristics;
+} IMAGE_FILE_HEADER, *PIMAGE_FILE_HEADER;
+#endif
+
+
 static MTYPE CheckMapType(LPCTSTR szFileName, LPBYTE pbHeaderBuffer, size_t cbHeaderBuffer)
 {
     LPDWORD HeaderInt32 = (LPDWORD)pbHeaderBuffer;
     LPCTSTR szExtension;
+    MTYPE TypeByExtension = MapTypeNotRecognized;
 
     // Don't do any checks if there is not at least 16 bytes
     if(cbHeaderBuffer > 0x10)
@@ -35,21 +61,34 @@ static MTYPE CheckMapType(LPCTSTR szFileName, LPBYTE pbHeaderBuffer, size_t cbHe
         DWORD DwordValue2 = BSWAP_INT32_UNSIGNED(HeaderInt32[2]);
         DWORD DwordValue3 = BSWAP_INT32_UNSIGNED(HeaderInt32[3]);
 
-        // Test for AVI files (Warcraft III cinematics) - 'RIFF', 'AVI ' or 'LIST'
-        if(DwordValue0 == 0x46464952 && DwordValue2 == 0x20495641 && DwordValue3 == 0x5453494C)
-            return MapTypeAviFile;
-
-        // Check for Starcraft II maps
+        // Check maps by extension (Starcraft, Starcraft II). We must do this before
+        // checking actual data, because the "NP_Protect" protector places
+        // fake Warcraft III header into the Starcraft II maps
         if((szExtension = _tcsrchr(szFileName, _T('.'))) != NULL)
         {
-            // The "NP_Protect" protector places fake Warcraft III header
-            // into the Starcraft II maps, whilst SC2 maps have no other header but MPQ v4
+            // Check for Starcraft I maps by extension
+            if(!_tcsicmp(szExtension, _T(".scm")) || !_tcsicmp(szExtension, _T(".scx")))
+            {
+                return MapTypeStarcraft;
+            }
+
+            // Check for Starcraft II maps by extension
             if(!_tcsicmp(szExtension, _T(".s2ma")) || !_tcsicmp(szExtension, _T(".SC2Map")) || !_tcsicmp(szExtension, _T(".SC2Mod")))
             {
                 return MapTypeStarcraft2;
             }
+
+            // Check for Warcraft III maps by extension
+            if(!_tcsicmp(szExtension, _T(".w3m")) || !_tcsicmp(szExtension, _T(".w3x")))
+            {
+                TypeByExtension = MapTypeWarcraft3;
+            }
         }
 
+        // Test for AVI files (Warcraft III cinematics) - 'RIFF', 'AVI ' or 'LIST'
+        if(DwordValue0 == 0x46464952 && DwordValue2 == 0x20495641 && DwordValue3 == 0x5453494C)
+            return MapTypeAviFile;
+
         // Check for Warcraft III maps
         if(DwordValue0 == 0x57334D48 && DwordValue1 == 0x00000000)
             return MapTypeWarcraft3;
@@ -57,15 +96,44 @@ static MTYPE CheckMapType(LPCTSTR szFileName, LPBYTE pbHeaderBuffer, size_t cbHe
 
     // MIX files are DLL files that contain MPQ in overlay.
     // Only Warcraft III is able to load them, so we consider them Warcraft III maps
-    if(cbHeaderBuffer > 0x200 && pbHeaderBuffer[0] == 'M' && pbHeaderBuffer[1] == 'Z')
+    // Do not include EXE files, because they may be World of Warcraft patches
+    if(cbHeaderBuffer > sizeof(IMAGE_DOS_HEADER))
     {
-        // Check the value of IMAGE_DOS_HEADER::e_lfanew at offset 0x3C
-        if(0 < HeaderInt32[0x0F] && HeaderInt32[0x0F] < 0x10000)
-            return MapTypeWarcraft3;
+        PIMAGE_FILE_HEADER pFileHeader;
+        PIMAGE_DOS_HEADER pDosHeader = (PIMAGE_DOS_HEADER)(pbHeaderBuffer);
+        size_t dwMaxAllowedSize = cbHeaderBuffer - sizeof(DWORD) - sizeof(IMAGE_FILE_HEADER);
+
+        // Verify the header of EXE/DLL files
+        if((pDosHeader->e_magic == IMAGE_DOS_SIGNATURE) && (0 < pDosHeader->e_lfanew && pDosHeader->e_lfanew < 0x10000))
+        {
+            // Is the file an EXE?
+            if((size_t)pDosHeader->e_lfanew <= dwMaxAllowedSize)
+            {
+                pFileHeader = (PIMAGE_FILE_HEADER)(pbHeaderBuffer + pDosHeader->e_lfanew + sizeof(DWORD));
+                if(pFileHeader->Characteristics & IMAGE_FILE_DLL)
+                {
+                    return MapTypeWarcraft3;
+                }
+            }
+        }
     }
 
     // No special map type recognized
-    return MapTypeNotRecognized;
+    return TypeByExtension;
+}
+
+static bool IsStarcraftBetaArchive(TMPQHeader * pHeader)
+{
+    // The archive must be version 1, with a standard header size
+    if(pHeader->dwID == ID_MPQ && pHeader->dwHeaderSize == MPQ_HEADER_SIZE_V1)
+    {
+        // Check for known archive sizes
+        return (pHeader->dwArchiveSize == 0x00028FB3 ||     // patch_rt.mpq
+                pHeader->dwArchiveSize == 0x0351853D ||     // StarDat.mpq
+                pHeader->dwArchiveSize == 0x0AEC8960);      // INSTALL.exe
+
+    }
+    return false;
 }
 
 static TMPQUserData * IsValidMpqUserData(ULONGLONG ByteOffset, ULONGLONG FileSize, void * pvUserData)
@@ -96,7 +164,7 @@ static TMPQUserData * IsValidMpqUserData(ULONGLONG ByteOffset, ULONGLONG FileSiz
 }
 
 // This function gets the right positions of the hash table and the block table.
-static int VerifyMpqTablePositions(TMPQArchive * ha, ULONGLONG FileSize)
+static DWORD VerifyMpqTablePositions(TMPQArchive * ha, ULONGLONG FileSize)
 {
     TMPQHeader * pHeader = ha->pHeader;
     ULONGLONG ByteOffset;
@@ -135,20 +203,49 @@ static int VerifyMpqTablePositions(TMPQArchive * ha, ULONGLONG FileSize)
     }
 
     // Check the begin of hi-block table
-    if(pHeader->HiBlockTablePos64 != 0)
-    {
-        ByteOffset = ha->MpqPos + pHeader->HiBlockTablePos64;
-        if(ByteOffset > FileSize)
-            return ERROR_BAD_FORMAT;
-    }
+    //if(pHeader->HiBlockTablePos64 != 0)
+    //{
+    //    ByteOffset = ha->MpqPos + pHeader->HiBlockTablePos64;
+    //    if(ByteOffset > FileSize)
+    //        return ERROR_BAD_FORMAT;
+    //}
 
     // All OK.
     return ERROR_SUCCESS;
 }
 
-/*****************************************************************************/
-/* Public functions                                                          */
-/*****************************************************************************/
+//-----------------------------------------------------------------------------
+// Support for alternate markers. Call before opening an archive
+
+#define SFILE_MARKERS_MIN_SIZE   (sizeof(DWORD) + sizeof(DWORD) + sizeof(const char *) + sizeof(const char *))
+
+bool WINAPI SFileSetArchiveMarkers(PSFILE_MARKERS pMarkers)
+{
+    // Check structure minimum size
+    if(pMarkers == NULL || pMarkers->dwSize < SFILE_MARKERS_MIN_SIZE)
+    {
+        SErrSetLastError(ERROR_INVALID_PARAMETER);
+        return false;
+    }
+
+    // Make sure that the MPQ cryptography is initialized at this time
+    InitializeMpqCryptography();
+
+    // Remember the marker for MPQ header
+    if(pMarkers->dwSignature != 0)
+        g_dwMpqSignature = pMarkers->dwSignature;
+
+    // Remember the encryption key for hash table
+    if(pMarkers->szHashTableKey != NULL)
+        g_dwHashTableKey = HashString(pMarkers->szHashTableKey, MPQ_HASH_FILE_KEY);
+
+    // Remember the encryption key for block table
+    if(pMarkers->szBlockTableKey != NULL)
+        g_dwBlockTableKey = HashString(pMarkers->szBlockTableKey, MPQ_HASH_FILE_KEY);
+
+    // Succeeded
+    return true;
+}
 
 //-----------------------------------------------------------------------------
 // SFileGetLocale and SFileSetLocale
@@ -156,13 +253,12 @@ static int VerifyMpqTablePositions(TMPQArchive * ha, ULONGLONG FileSize)
 
 LCID WINAPI SFileGetLocale()
 {
-    return lcFileLocale;
+    return g_lcFileLocale;
 }
 
-LCID WINAPI SFileSetLocale(LCID lcNewLocale)
+LCID WINAPI SFileSetLocale(LCID lcFileLocale)
 {
-    lcFileLocale = lcNewLocale;
-    return lcFileLocale;
+    return (g_lcFileLocale = lcFileLocale);
 }
 
 //-----------------------------------------------------------------------------
@@ -179,20 +275,20 @@ bool WINAPI SFileOpenArchive(
     DWORD dwFlags,
     HANDLE * phMpq)
 {
-    TMPQUserData * pUserData;
+    TMPQUserData * pUserData = NULL;
     TFileStream * pStream = NULL;       // Open file stream
     TMPQArchive * ha = NULL;            // Archive handle
     TFileEntry * pFileEntry;
     ULONGLONG FileSize = 0;             // Size of the file
     LPBYTE pbHeaderBuffer = NULL;       // Buffer for searching MPQ header
     DWORD dwStreamFlags = (dwFlags & STREAM_FLAGS_MASK);
-    MTYPE MapType = MapTypeNotRecognized;
-    int nError = ERROR_SUCCESS;   
+    MTYPE MapType = MapTypeNotChecked;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // Verify the parameters
     if(szMpqName == NULL || *szMpqName == 0 || phMpq == NULL)
     {
-        SetLastError(ERROR_INVALID_PARAMETER);
+        SErrSetLastError(ERROR_INVALID_PARAMETER);
         return false;
     }
 
@@ -209,39 +305,41 @@ bool WINAPI SFileOpenArchive(
         return false;
 
     // Check the file size. There must be at least 0x20 bytes
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
         FileStream_GetSize(pStream, &FileSize);
         if(FileSize < MPQ_HEADER_SIZE_V1)
-            nError = ERROR_BAD_FORMAT;
+            dwErrCode = ERROR_BAD_FORMAT;
     }
 
     // Allocate the MPQhandle
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
-        if((ha = STORM_ALLOC(TMPQArchive, 1)) == NULL)
-            nError = ERROR_NOT_ENOUGH_MEMORY;
+        if((ha = STORM_ALLOC(TMPQArchive, 1)) != NULL)
+            memset(ha, 0, sizeof(TMPQArchive));
+        else
+            dwErrCode = ERROR_NOT_ENOUGH_MEMORY;
     }
 
     // Allocate buffer for searching MPQ header
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
         pbHeaderBuffer = STORM_ALLOC(BYTE, HEADER_SEARCH_BUFFER_SIZE);
         if(pbHeaderBuffer == NULL)
-            nError = ERROR_NOT_ENOUGH_MEMORY;
+            dwErrCode = ERROR_NOT_ENOUGH_MEMORY;
     }
 
     // Find the position of MPQ header
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
-        ULONGLONG SearchOffset = 0;
+        ULONGLONG ByteOffset = 0;
         ULONGLONG EndOfSearch = FileSize;
         DWORD dwStrmFlags = 0;
         DWORD dwHeaderSize;
         DWORD dwHeaderID;
         bool bSearchComplete = false;
 
-        memset(ha, 0, sizeof(TMPQArchive));
+        ha->dwValidFileFlags = MPQ_FILE_VALID_FLAGS;
         ha->pfnHashString = HashStringSlash;
         ha->pStream = pStream;
         pStream = NULL;
@@ -255,43 +353,44 @@ bool WINAPI SFileOpenArchive(
 
         // Also remember if this MPQ is a patch
         ha->dwFlags |= (dwFlags & MPQ_OPEN_PATCH) ? MPQ_FLAG_PATCH : 0;
-       
+
         // Limit the header searching to about 130 MB of data
         if(EndOfSearch > 0x08000000)
             EndOfSearch = 0x08000000;
+        if(FileSize < HEADER_SEARCH_BUFFER_SIZE)
+            memset(pbHeaderBuffer, 0, HEADER_SEARCH_BUFFER_SIZE);
 
         // Find the offset of MPQ header within the file
-        while(bSearchComplete == false && SearchOffset < EndOfSearch)
+        while(bSearchComplete == false && ByteOffset < EndOfSearch)
         {
             // Always read at least 0x1000 bytes for performance.
             // This is what Storm.dll (2002) does.
             DWORD dwBytesAvailable = HEADER_SEARCH_BUFFER_SIZE;
-            DWORD dwInBufferOffset = 0;
 
             // Cut the bytes available, if needed
-            if((FileSize - SearchOffset) < HEADER_SEARCH_BUFFER_SIZE)
-                dwBytesAvailable = (DWORD)(FileSize - SearchOffset);
+            if((FileSize - ByteOffset) < HEADER_SEARCH_BUFFER_SIZE)
+                dwBytesAvailable = (DWORD)(FileSize - ByteOffset);
 
             // Read the eventual MPQ header
-            if(!FileStream_Read(ha->pStream, &SearchOffset, pbHeaderBuffer, dwBytesAvailable))
+            if(!FileStream_Read(ha->pStream, &ByteOffset, pbHeaderBuffer, dwBytesAvailable))
             {
-                nError = GetLastError();
+                dwErrCode = SErrGetLastError();
                 break;
             }
 
             // Check whether the file is AVI file or a Warcraft III/Starcraft II map
-            if(SearchOffset == 0)
+            if(MapType == MapTypeNotChecked)
             {
                 // Do nothing if the file is an AVI file
                 if((MapType = CheckMapType(szMpqName, pbHeaderBuffer, dwBytesAvailable)) == MapTypeAviFile)
                 {
-                    nError = ERROR_AVI_FILE;
+                    dwErrCode = ERROR_AVI_FILE;
                     break;
                 }
             }
 
             // Search the header buffer
-            while(dwInBufferOffset < dwBytesAvailable)
+            for(DWORD dwInBufferOffset = 0; dwInBufferOffset < dwBytesAvailable; dwInBufferOffset += 0x200)
             {
                 // Copy the data from the potential header buffer to the MPQ header
                 memcpy(ha->HeaderData, pbHeaderBuffer + dwInBufferOffset, sizeof(ha->HeaderData));
@@ -303,18 +402,25 @@ bool WINAPI SFileOpenArchive(
                 {
                     if(ha->pUserData == NULL && dwHeaderID == ID_MPQ_USERDATA)
                     {
+                        // Copy the eventual user data to the separate buffer
+                        memcpy(&ha->UserData, ha->HeaderData, sizeof(TMPQUserData));
+
                         // Verify if this looks like a valid user data
-                        pUserData = IsValidMpqUserData(SearchOffset, FileSize, ha->HeaderData);
+                        pUserData = IsValidMpqUserData(ByteOffset, FileSize, &ha->UserData);
                         if(pUserData != NULL)
                         {
-                            // Fill the user data header
-                            ha->UserDataPos = SearchOffset;
-                            ha->pUserData = &ha->UserData;
-                            memcpy(ha->pUserData, pUserData, sizeof(TMPQUserData));
+                            // Set the byte offset to the loaded user data
+                            ULONGLONG TempByteOffset = ByteOffset + pUserData->dwHeaderOffs;
 
-                            // Continue searching from that position
-                            SearchOffset += ha->pUserData->dwHeaderOffs;
-                            break;
+                            // Read the eventual MPQ header from the position where the user data points
+                            if(!FileStream_Read(ha->pStream, &TempByteOffset, ha->HeaderData, sizeof(ha->HeaderData)))
+                            {
+                                dwErrCode = SErrGetLastError();
+                                break;
+                            }
+
+                            // Re-initialize the header ID
+                            dwHeaderID = BSWAP_INT32_UNSIGNED(ha->HeaderData[0]);
                         }
                     }
                 }
@@ -324,11 +430,11 @@ bool WINAPI SFileOpenArchive(
                 // Abused by Spazzler Map protector. Note that the size check is not present
                 // in Storm.dll v 1.00, so Diablo I code would load the MPQ anyway.
                 dwHeaderSize = BSWAP_INT32_UNSIGNED(ha->HeaderData[1]);
-                if(dwHeaderID == ID_MPQ && dwHeaderSize >= MPQ_HEADER_SIZE_V1)
+                if(dwHeaderID == g_dwMpqSignature && dwHeaderSize >= MPQ_HEADER_SIZE_V1)
                 {
                     // Now convert the header to version 4
-                    nError = ConvertMpqHeaderToFormat4(ha, SearchOffset, FileSize, dwFlags, MapType);
-                    if(nError != ERROR_FAKE_MPQ_HEADER)
+                    dwErrCode = ConvertMpqHeaderToFormat4(ha, ByteOffset, FileSize, dwFlags, MapType);
+                    if(dwErrCode != ERROR_FAKE_MPQ_HEADER)
                     {
                         bSearchComplete = true;
                         break;
@@ -336,10 +442,10 @@ bool WINAPI SFileOpenArchive(
                 }
 
                 // Check for MPK archives (Longwu Online - MPQ fork)
-                if(MapType == MapTypeNotRecognized && dwHeaderID == ID_MPK)
+                if(MapType == MapTypeNotRecognized && (dwFlags & MPQ_OPEN_FORCE_MPQ_V1) == 0 && dwHeaderID == ID_MPK)
                 {
                     // Now convert the MPK header to MPQ Header version 4
-                    nError = ConvertMpkHeaderToFormat4(ha, FileSize, dwFlags);
+                    dwErrCode = ConvertMpkHeaderToFormat4(ha, FileSize, dwFlags);
                     bSearchComplete = true;
                     break;
                 }
@@ -347,37 +453,48 @@ bool WINAPI SFileOpenArchive(
                 // If searching for the MPQ header is disabled, return an error
                 if(dwFlags & MPQ_OPEN_NO_HEADER_SEARCH)
                 {
-                    nError = ERROR_NOT_SUPPORTED;
+                    dwErrCode = ERROR_NOT_SUPPORTED;
                     bSearchComplete = true;
                     break;
                 }
 
                 // Move the pointers
-                SearchOffset += 0x200;
-                dwInBufferOffset += 0x200;
+                ByteOffset += 0x200;
+                pUserData = NULL;
             }
         }
 
         // Did we identify one of the supported headers?
-        if(nError == ERROR_SUCCESS)
+        if(dwErrCode == ERROR_SUCCESS)
         {
+            // If we retrieved the offset from the user data offset, initialize the user data
+            if(pUserData != NULL)
+            {
+                // Fill the user data header
+                ha->pUserData = &ha->UserData;
+                ha->UserDataPos = ByteOffset;
+
+                // Set the real byte offset
+                ByteOffset = ByteOffset + pUserData->dwHeaderOffs;
+            }
+
             // Set the user data position to the MPQ header, if none
             if(ha->pUserData == NULL)
-                ha->UserDataPos = SearchOffset;
+                ha->UserDataPos = ByteOffset;
 
             // Set the position of the MPQ header
             ha->pHeader  = (TMPQHeader *)ha->HeaderData;
-            ha->MpqPos   = SearchOffset;
+            ha->MpqPos   = ByteOffset;
             ha->FileSize = FileSize;
 
             // Sector size must be nonzero.
-            if(SearchOffset >= FileSize || ha->pHeader->wSectorSize == 0)
-                nError = ERROR_BAD_FORMAT;
+            if(ByteOffset >= FileSize || ha->pHeader->wSectorSize == 0)
+                dwErrCode = ERROR_BAD_FORMAT;
         }
     }
 
     // Fix table positions according to format
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
         // Dump the header
 //      DumpMpqHeader(ha->pHeader);
@@ -409,33 +526,56 @@ bool WINAPI SFileOpenArchive(
         if(dwFlags & MPQ_OPEN_FORCE_LISTFILE)
             ha->dwFlags |= MPQ_FLAG_LISTFILE_FORCE;
 
-        // Remember whether whis is a map for Warcraft III
-        if(MapType == MapTypeWarcraft3)
-            ha->dwFlags |= MPQ_FLAG_WAR3_MAP;
+        // StarDat.mpq from Starcraft I BETA: Enable special compression types
+        if(IsStarcraftBetaArchive(ha->pHeader))
+            ha->dwFlags |= MPQ_FLAG_STARCRAFT_BETA;
 
-        // Set the size of file sector
-        ha->dwSectorSize = (0x200 << ha->pHeader->wSectorSize);
+        // Set the mask for the file offset. In MPQs version 1,
+        // all offsets are 32-bit and overflow is allowed.
+        // For MPQs v2+, file offset if 64-bit.
+        ha->FileOffsetMask = GetFileOffsetMask(ha);
 
-        // Verify if any of the tables doesn't start beyond the end of the file
-        nError = VerifyMpqTablePositions(ha, FileSize);
+        // Maps from StarCraft and Warcraft III need special treatment
+        switch(MapType)
+        {
+            case MapTypeStarcraft:
+                ha->dwValidFileFlags = MPQ_FILE_VALID_FLAGS_SCX;
+                ha->dwFlags |= MPQ_FLAG_STARCRAFT;
+                break;
+
+            case MapTypeWarcraft3:
+                ha->dwValidFileFlags = MPQ_FILE_VALID_FLAGS_W3X;
+                ha->dwFlags |= MPQ_FLAG_WAR3_MAP;
+                break;
+        }
+
+        // Set the size of file sector. Be sure to check for integer overflow
+        if((ha->dwSectorSize = (0x200 << ha->pHeader->wSectorSize)) == 0)
+            dwErrCode = ERROR_FILE_CORRUPT;
+    }
+
+    // Verify if any of the tables doesn't start beyond the end of the file
+    if(dwErrCode == ERROR_SUCCESS)
+    {
+        dwErrCode = VerifyMpqTablePositions(ha, FileSize);
     }
 
     // Read the hash table. Ignore the result, as hash table is no longer required
     // Read HET table. Ignore the result, as HET table is no longer required
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
-        nError = LoadAnyHashTable(ha);
+        dwErrCode = LoadAnyHashTable(ha);
     }
 
     // Now, build the file table. It will be built by combining
     // the block table, BET table, hi-block table, (attributes) and (listfile).
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
-        nError = BuildFileTable(ha);
+        dwErrCode = BuildFileTable(ha);
     }
 
     // Load the internal listfile and include it to the file table
-    if(nError == ERROR_SUCCESS && (dwFlags & MPQ_OPEN_NO_LISTFILE) == 0)
+    if(dwErrCode == ERROR_SUCCESS && (dwFlags & MPQ_OPEN_NO_LISTFILE) == 0)
     {
         // Quick check for (listfile)
         pFileEntry = GetFileEntryLocale(ha, LISTFILE_NAME, LANG_NEUTRAL);
@@ -448,7 +588,7 @@ bool WINAPI SFileOpenArchive(
     }
 
     // Load the "(attributes)" file and merge it to the file table
-    if(nError == ERROR_SUCCESS && (dwFlags & MPQ_OPEN_NO_ATTRIBUTES) == 0 && (ha->dwFlags & MPQ_FLAG_BLOCK_TABLE_CUT) == 0)
+    if(dwErrCode == ERROR_SUCCESS && (dwFlags & MPQ_OPEN_NO_ATTRIBUTES) == 0 && (ha->dwFlags & MPQ_FLAG_BLOCK_TABLE_CUT) == 0)
     {
         // Quick check for (attributes)
         pFileEntry = GetFileEntryLocale(ha, ATTRIBUTES_NAME, LANG_NEUTRAL);
@@ -461,7 +601,7 @@ bool WINAPI SFileOpenArchive(
     }
 
     // Remember whether the archive has weak signature. Only for MPQs format 1.0.
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
         // Quick check for (signature)
         pFileEntry = GetFileEntryLocale(ha, SIGNATURE_NAME, LANG_NEUTRAL);
@@ -477,11 +617,11 @@ bool WINAPI SFileOpenArchive(
     }
 
     // Cleanup and exit
-    if(nError != ERROR_SUCCESS)
+    if(dwErrCode != ERROR_SUCCESS)
     {
         FileStream_Close(pStream);
         FreeArchiveHandle(ha);
-        SetLastError(nError);
+        SErrSetLastError(dwErrCode);
         ha = NULL;
     }
 
@@ -490,7 +630,7 @@ bool WINAPI SFileOpenArchive(
         STORM_FREE(pbHeaderBuffer);
     if(phMpq != NULL)
         *phMpq = ha;
-    return (nError == ERROR_SUCCESS);
+    return (dwErrCode == ERROR_SUCCESS);
 }
 
 //-----------------------------------------------------------------------------
@@ -506,7 +646,7 @@ bool WINAPI SFileSetDownloadCallback(HANDLE hMpq, SFILE_DOWNLOAD_CALLBACK Downlo
     // Do nothing if 'hMpq' is bad parameter
     if(!IsValidMpqHandle(hMpq))
     {
-        SetLastError(ERROR_INVALID_HANDLE);
+        SErrSetLastError(ERROR_INVALID_HANDLE);
         return false;
     }
 
@@ -525,13 +665,13 @@ bool WINAPI SFileSetDownloadCallback(HANDLE hMpq, SFILE_DOWNLOAD_CALLBACK Downlo
 bool WINAPI SFileFlushArchive(HANDLE hMpq)
 {
     TMPQArchive * ha;
-    int nResultError = ERROR_SUCCESS;
-    int nError;
+    DWORD dwResultError = ERROR_SUCCESS;
+    DWORD dwErrCode;
 
     // Do nothing if 'hMpq' is bad parameter
     if((ha = IsValidMpqHandle(hMpq)) == NULL)
     {
-        SetLastError(ERROR_INVALID_HANDLE);
+        SErrSetLastError(ERROR_INVALID_HANDLE);
         return false;
     }
 
@@ -551,23 +691,23 @@ bool WINAPI SFileFlushArchive(HANDLE hMpq)
 
         if(ha->dwFlags & MPQ_FLAG_SIGNATURE_NEW)
         {
-            nError = SSignFileCreate(ha);
-            if(nError != ERROR_SUCCESS)
-                nResultError = nError;
+            dwErrCode = SSignFileCreate(ha);
+            if(dwErrCode != ERROR_SUCCESS)
+                dwResultError = dwErrCode;
         }
 
         if(ha->dwFlags & (MPQ_FLAG_LISTFILE_NEW | MPQ_FLAG_LISTFILE_FORCE))
         {
-            nError = SListFileSaveToMpq(ha);
-            if(nError != ERROR_SUCCESS)
-                nResultError = nError;
+            dwErrCode = SListFileSaveToMpq(ha);
+            if(dwErrCode != ERROR_SUCCESS)
+                dwResultError = dwErrCode;
         }
 
         if(ha->dwFlags & MPQ_FLAG_ATTRIBUTES_NEW)
         {
-            nError = SAttrFileSaveToMpq(ha);
-            if(nError != ERROR_SUCCESS)
-                nResultError = nError;
+            dwErrCode = SAttrFileSaveToMpq(ha);
+            if(dwErrCode != ERROR_SUCCESS)
+                dwResultError = dwErrCode;
         }
 
         // Save HET table, BET table, hash table, block table, hi-block table
@@ -578,16 +718,16 @@ bool WINAPI SFileFlushArchive(HANDLE hMpq)
                 RebuildHetTable(ha);
 
             // Save all MPQ tables first
-            nError = SaveMPQTables(ha);
-            if(nError != ERROR_SUCCESS)
-                nResultError = nError;
+            dwErrCode = SaveMPQTables(ha);
+            if(dwErrCode != ERROR_SUCCESS)
+                dwResultError = dwErrCode;
 
             // If the archive has weak signature, we need to finish it
             if(ha->dwFileFlags3 != 0)
             {
-                nError = SSignFileFinish(ha);
-                if(nError != ERROR_SUCCESS)
-                    nResultError = nError;
+                dwErrCode = SSignFileFinish(ha);
+                if(dwErrCode != ERROR_SUCCESS)
+                    dwResultError = dwErrCode;
             }
         }
 
@@ -596,9 +736,9 @@ bool WINAPI SFileFlushArchive(HANDLE hMpq)
     }
 
     // Return the error
-    if(nResultError != ERROR_SUCCESS)
-        SetLastError(nResultError);
-    return (nResultError == ERROR_SUCCESS);
+    if(dwResultError != ERROR_SUCCESS)
+        SErrSetLastError(dwResultError);
+    return (dwResultError == ERROR_SUCCESS);
 }
 
 //-----------------------------------------------------------------------------
@@ -613,7 +753,7 @@ bool WINAPI SFileCloseArchive(HANDLE hMpq)
     // Only if the handle is valid
     if(ha == NULL)
     {
-        SetLastError(ERROR_INVALID_HANDLE);
+        SErrSetLastError(ERROR_INVALID_HANDLE);
         return false;
     }
 
