@@ -13,6 +13,11 @@
 #include "StormLib.h"
 #include "StormCommon.h"
 
+//-----------------------------------------------------------------------------
+// External references (not public functions)
+
+int WINAPI SCompDecompressX(TMPQArchive * ha, void * pvOutBuffer, int * pcbOutBuffer, void * pbInBuffer, int cbInBuffer);
+
 //-----------------------------------------------------------------------------
 // Local functions
 
@@ -21,7 +26,7 @@
 //  dwByteOffset  - Position of sector in the file (relative to file begin)
 //  dwBytesToRead - Number of bytes to read. Must be multiplier of sector size.
 //  pdwBytesRead  - Stored number of bytes loaded
-static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
 {
     ULONGLONG RawFilePos;
     TMPQArchive * ha = hf->ha;
@@ -35,7 +40,7 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
     DWORD dwSectorIndex = dwByteOffset / ha->dwSectorSize;
     DWORD dwSectorsDone = 0;
     DWORD dwBytesRead = 0;
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // Note that dwByteOffset must be aligned to size of one sector
     // Note that dwBytesToRead must be a multiplier of one sector size
@@ -53,9 +58,9 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
         // If the sector positions are not loaded yet, do it
         if(hf->SectorOffsets == NULL)
         {
-            nError = AllocateSectorOffsets(hf, true);
-            if(nError != ERROR_SUCCESS || hf->SectorOffsets == NULL)
-                return nError;
+            dwErrCode = AllocateSectorOffsets(hf, true);
+            if(dwErrCode != ERROR_SUCCESS || hf->SectorOffsets == NULL)
+                return dwErrCode;
         }
 
         // If the sector checksums are not loaded yet, load them now.
@@ -68,17 +73,17 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
             // or not, we won't try that again for the given file.
             //
 
-            AllocateSectorChecksums(hf, true);
-            hf->bLoadedSectorCRCs = true;
+            if(AllocateSectorChecksums(hf, true) == ERROR_SUCCESS)
+                hf->bLoadedSectorCRCs = true;
         }
 
         // TODO: If the raw data MD5s are not loaded yet, load them now
         // Only do it if the MPQ is of format 4.0
 //      if(ha->pHeader->wFormatVersion >= MPQ_FORMAT_VERSION_4 && ha->pHeader->dwRawChunkSize != 0)
 //      {
-//          nError = AllocateRawMD5s(hf, true);
-//          if(nError != ERROR_SUCCESS)
-//              return nError;
+//          dwErrCode = AllocateRawMD5s(hf, true);
+//          if(dwErrCode != ERROR_SUCCESS)
+//              return dwErrCode;
 //      }
 
         // Assign the temporary buffer as target for read operation
@@ -126,7 +131,7 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
                     hf->dwFileKey = DetectFileKeyByContent(pbInSector, dwBytesInThisSector, hf->dwDataSize);
                     if(hf->dwFileKey == 0)
                     {
-                        nError = ERROR_UNKNOWN_FILE_KEY;
+                        dwErrCode = ERROR_UNKNOWN_FILE_KEY;
                         break;
                     }
                 }
@@ -148,7 +153,7 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
                     dwAdlerValue = adler32(0, pbInSector, dwRawBytesInThisSector);
                     if(dwAdlerValue != dwAdlerExpected)
                     {
-                        nError = ERROR_CHECKSUM_ERROR;
+                        dwErrCode = ERROR_CHECKSUM_ERROR;
                         break;
                     }
                 }
@@ -159,34 +164,47 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
             // by comparing uncompressed and compressed size !!!
             if(dwRawBytesInThisSector < dwBytesInThisSector)
             {
-                int cbOutSector = dwBytesInThisSector;
-                int cbInSector = dwRawBytesInThisSector;
-                int nResult = 0;
-
-                // Is the file compressed by Blizzard's multiple compression ?
-                if(pFileEntry->dwFlags & MPQ_FILE_COMPRESS)
+                if(dwRawBytesInThisSector != 0)
                 {
-                    // Remember the last used compression
-                    hf->dwCompression0 = pbInSector[0];
+                    int cbOutSector = dwBytesInThisSector;
+                    int cbInSector = dwRawBytesInThisSector;
+                    int nResult = 0;
 
-                    // Decompress the data
-                    if(ha->pHeader->wFormatVersion >= MPQ_FORMAT_VERSION_2)
-                        nResult = SCompDecompress2(pbOutSector, &cbOutSector, pbInSector, cbInSector);
-                    else
-                        nResult = SCompDecompress(pbOutSector, &cbOutSector, pbInSector, cbInSector);
+                    // Is the file compressed by Blizzard's multiple compression ?
+                    if(pFileEntry->dwFlags & MPQ_FILE_COMPRESS)
+                    {
+                        // Remember the last used compression
+                        hf->dwCompression0 = pbInSector[0];
+
+                        // Decompress the data. We need to perform MPQ-specific decompression,
+                        // as multiple Blizzard games may have their own decompression tables
+                        // and even decompression methods.
+                        nResult = SCompDecompressX(ha, pbOutSector, &cbOutSector, pbInSector, cbInSector);
+                    }
+
+                    // Is the file compressed by PKWARE Data Compression Library ?
+                    else if(pFileEntry->dwFlags & MPQ_FILE_IMPLODE)
+                    {
+                        nResult = SCompExplode(pbOutSector, &cbOutSector, pbInSector, cbInSector);
+                    }
+
+                    // Did the decompression fail ?
+                    if(nResult == 0)
+                    {
+                        dwErrCode = ERROR_FILE_CORRUPT;
+                        break;
+                    }
+
+                    // Special case (MPQ_2024_v1_300TK2.09p.w3x, file File00010254.blp):
+                    // Extracted less than required. Fill the rest with zeros
+                    if((DWORD)(cbOutSector) < dwBytesInThisSector)
+                    {
+                        memset(pbOutSector + cbOutSector, 0, dwBytesInThisSector - cbOutSector);
+                    }
                 }
-
-                // Is the file compressed by PKWARE Data Compression Library ?
-                else if(pFileEntry->dwFlags & MPQ_FILE_IMPLODE)
+                else
                 {
-                    nResult = SCompExplode(pbOutSector, &cbOutSector, pbInSector, cbInSector);
-                }
-
-                // Did the decompression fail ?
-                if(nResult == 0)
-                {
-                    nError = ERROR_FILE_CORRUPT;
-                    break;
+                    memset(pbOutSector, 0, dwBytesInThisSector);
                 }
             }
             else
@@ -206,33 +224,33 @@ static int ReadMpqSectors(TMPQFile * hf, LPBYTE pbBuffer, DWORD dwByteOffset, DW
     }
     else
     {
-        nError = GetLastError();
+        dwErrCode = SErrGetLastError();
     }
 
     // Free all used buffers
     if(pbRawSector != NULL)
         STORM_FREE(pbRawSector);
-    
+
     // Give the caller thenumber of bytes read
     *pdwBytesRead = dwBytesRead;
-    return nError; 
+    return dwErrCode;
 }
 
-static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
 {
     ULONGLONG RawFilePos = hf->RawFilePos;
     TMPQArchive * ha = hf->ha;
     TFileEntry * pFileEntry = hf->pFileEntry;
     LPBYTE pbCompressed = NULL;
     LPBYTE pbRawData;
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // If the file buffer is not allocated yet, do it.
     if(hf->pbFileSector == NULL)
     {
-        nError = AllocateSectorBuffer(hf);
-        if(nError != ERROR_SUCCESS || hf->pbFileSector == NULL)
-            return nError;
+        dwErrCode = AllocateSectorBuffer(hf);
+        if(dwErrCode != ERROR_SUCCESS || hf->pbFileSector == NULL)
+            return dwErrCode;
     }
 
     // If the file is a patch file, adjust raw data offset
@@ -243,6 +261,8 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     // If the file sector is not loaded yet, do it
     if(hf->dwSectorOffs != 0)
     {
+        DWORD cbRawData = hf->dwDataSize;
+
         // Is the file compressed?
         if(pFileEntry->dwFlags & MPQ_FILE_COMPRESS_MASK)
         {
@@ -250,14 +270,17 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
             pbCompressed = STORM_ALLOC(BYTE, pFileEntry->dwCmpSize);
             if(pbCompressed == NULL)
                 return ERROR_NOT_ENOUGH_MEMORY;
+            
+            // Redirect reading
             pbRawData = pbCompressed;
+            cbRawData = pFileEntry->dwCmpSize;
         }
-        
+
         // Load the raw (compressed, encrypted) data
-        if(!FileStream_Read(ha->pStream, &RawFilePos, pbRawData, pFileEntry->dwCmpSize))
+        if(!FileStream_Read(ha->pStream, &RawFilePos, pbRawData, cbRawData))
         {
             STORM_FREE(pbCompressed);
-            return GetLastError();
+            return SErrGetLastError();
         }
 
         // If the file is encrypted, we have to decrypt the data first
@@ -285,9 +308,9 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
             // --------------------------------------  ---------- -------- -------- ---------------
             // esES\DBFilesClient\LightSkyBox.dbc      0xBE->0xA2  0xBC     0xBC     Yes
             // deDE\DBFilesClient\MountCapability.dbc  0x93->0x77  0x77     0x77     No
-            // 
+            //
 
-            if(pFileEntry->dwFlags & MPQ_FILE_PATCH_FILE)
+            if(pFileEntry->dwFlags & MPQ_FILE_PATCH_FILE && cbInBuffer > sizeof(TPatchInfo))
                 cbInBuffer = cbInBuffer - sizeof(TPatchInfo);
 
             // Is the file compressed by Blizzard's multiple compression ?
@@ -308,7 +331,7 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
             else if(pFileEntry->dwFlags & MPQ_FILE_IMPLODE)
                 nResult = SCompExplode(hf->pbFileSector, &cbOutBuffer, pbRawData, cbInBuffer);
 
-            nError = (nResult != 0) ? ERROR_SUCCESS : ERROR_FILE_CORRUPT;
+            dwErrCode = (nResult != 0) ? ERROR_SUCCESS : ERROR_FILE_CORRUPT;
         }
         else
         {
@@ -326,7 +349,7 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
 
     // At this moment, we have the file loaded into the file buffer.
     // Copy as much as the caller wants
-    if(nError == ERROR_SUCCESS && hf->dwSectorOffs == 0)
+    if(dwErrCode == ERROR_SUCCESS && hf->dwSectorOffs == 0)
     {
         // File position is greater or equal to file size ?
         if(dwFilePos >= hf->dwDataSize)
@@ -347,17 +370,17 @@ static int ReadMpqFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     }
 
     // An error, sorry
-    return nError;
+    return dwErrCode;
 }
 
-static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
 {
     ULONGLONG RawFilePos = hf->RawFilePos + 0x0C;   // For some reason, MPK files start at position (hf->RawFilePos + 0x0C)
     TMPQArchive * ha = hf->ha;
     TFileEntry * pFileEntry = hf->pFileEntry;
     LPBYTE pbCompressed = NULL;
     LPBYTE pbRawData = hf->pbFileSector;
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // We do not support patch files in MPK archives
     assert(hf->pPatchInfo == NULL);
@@ -365,9 +388,9 @@ static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     // If the file buffer is not allocated yet, do it.
     if(hf->pbFileSector == NULL)
     {
-        nError = AllocateSectorBuffer(hf);
-        if(nError != ERROR_SUCCESS || hf->pbFileSector == NULL)
-            return nError;
+        dwErrCode = AllocateSectorBuffer(hf);
+        if(dwErrCode != ERROR_SUCCESS || hf->pbFileSector == NULL)
+            return dwErrCode;
         pbRawData = hf->pbFileSector;
     }
 
@@ -383,12 +406,12 @@ static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
                 return ERROR_NOT_ENOUGH_MEMORY;
             pbRawData = pbCompressed;
         }
-        
+
         // Load the raw (compressed, encrypted) data
         if(!FileStream_Read(ha->pStream, &RawFilePos, pbRawData, pFileEntry->dwCmpSize))
         {
             STORM_FREE(pbCompressed);
-            return GetLastError();
+            return SErrGetLastError();
         }
 
         // If the file is encrypted, we have to decrypt the data first
@@ -397,14 +420,14 @@ static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
             DecryptMpkTable(pbRawData, pFileEntry->dwCmpSize);
         }
 
-        // If the file is compressed, we have to decompress it now
+        // If the file is compressed, we have to decompress it now                                      on 
         if(pFileEntry->dwFlags & MPQ_FILE_COMPRESS_MASK)
         {
             int cbOutBuffer = (int)hf->dwDataSize;
 
             hf->dwCompression0 = pbRawData[0];
             if(!SCompDecompressMpk(hf->pbFileSector, &cbOutBuffer, pbRawData, (int)pFileEntry->dwCmpSize))
-                nError = ERROR_FILE_CORRUPT;
+                dwErrCode = ERROR_FILE_CORRUPT;
         }
         else
         {
@@ -422,7 +445,7 @@ static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
 
     // At this moment, we have the file loaded into the file buffer.
     // Copy as much as the caller wants
-    if(nError == ERROR_SUCCESS && hf->dwSectorOffs == 0)
+    if(dwErrCode == ERROR_SUCCESS && hf->dwSectorOffs == 0)
     {
         // File position is greater or equal to file size ?
         if(dwFilePos >= hf->dwDataSize)
@@ -448,7 +471,7 @@ static int ReadMpkFileSingleUnit(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
 }
 
 
-static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwBytesToRead, LPDWORD pdwBytesRead)
 {
     TMPQArchive * ha = hf->ha;
     LPBYTE pbBuffer = (BYTE *)pvBuffer;
@@ -456,7 +479,7 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     DWORD dwSectorSizeMask = ha->dwSectorSize - 1;      // Mask for block size, usually 0x0FFF
     DWORD dwFileSectorPos;                              // File offset of the loaded sector
     DWORD dwBytesRead;                                  // Number of bytes read (temporary variable)
-    int nError;
+    DWORD dwErrCode;
 
     // If the file position is at or beyond end of file, do nothing
     if(dwFilePos >= hf->dwDataSize)
@@ -475,9 +498,9 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     // If the file sector buffer is not allocated yet, do it now
     if(hf->pbFileSector == NULL)
     {
-        nError = AllocateSectorBuffer(hf);
-        if(nError != ERROR_SUCCESS || hf->pbFileSector == NULL)
-            return nError;
+        dwErrCode = AllocateSectorBuffer(hf);
+        if(dwErrCode != ERROR_SUCCESS || hf->pbFileSector == NULL)
+            return dwErrCode;
     }
 
     // Load the first (incomplete) file sector
@@ -485,15 +508,15 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     {
         DWORD dwBytesInSector = ha->dwSectorSize;
         DWORD dwBufferOffs = dwFilePos & dwSectorSizeMask;
-        DWORD dwToCopy;                                     
+        DWORD dwToCopy;
 
         // Is the file sector already loaded ?
         if(hf->dwSectorOffs != dwFileSectorPos)
         {
             // Load one MPQ sector into archive buffer
-            nError = ReadMpqSectors(hf, hf->pbFileSector, dwFileSectorPos, ha->dwSectorSize, &dwBytesInSector);
-            if(nError != ERROR_SUCCESS)
-                return nError;
+            dwErrCode = ReadMpqSectors(hf, hf->pbFileSector, dwFileSectorPos, ha->dwSectorSize, &dwBytesInSector);
+            if(dwErrCode != ERROR_SUCCESS)
+                return dwErrCode;
 
             // Remember that the data loaded to the sector have new file offset
             hf->dwSectorOffs = dwFileSectorPos;
@@ -525,9 +548,9 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
         DWORD dwBlockBytes = dwBytesToRead & ~dwSectorSizeMask;
 
         // Load all sectors to the output buffer
-        nError = ReadMpqSectors(hf, pbBuffer, dwFileSectorPos, dwBlockBytes, &dwBytesRead);
-        if(nError != ERROR_SUCCESS)
-            return nError;
+        dwErrCode = ReadMpqSectors(hf, pbBuffer, dwFileSectorPos, dwBlockBytes, &dwBytesRead);
+        if(dwErrCode != ERROR_SUCCESS)
+            return dwErrCode;
 
         // Update pointers
         dwTotalBytesRead += dwBytesRead;
@@ -545,9 +568,9 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
         if(hf->dwSectorOffs != dwFileSectorPos)
         {
             // Load one MPQ sector into archive buffer
-            nError = ReadMpqSectors(hf, hf->pbFileSector, dwFileSectorPos, ha->dwSectorSize, &dwBytesRead);
-            if(nError != ERROR_SUCCESS)
-                return nError;
+            dwErrCode = ReadMpqSectors(hf, hf->pbFileSector, dwFileSectorPos, ha->dwSectorSize, &dwBytesRead);
+            if(dwErrCode != ERROR_SUCCESS)
+                return dwErrCode;
 
             // Remember that the data loaded to the sector have new file offset
             hf->dwSectorOffs = dwFileSectorPos;
@@ -559,7 +582,7 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
 
         // Copy the data from the cached last sector to the caller's buffer
         memcpy(pbBuffer, hf->pbFileSector, dwToCopy);
-        
+
         // Update pointers
         dwTotalBytesRead += dwToCopy;
     }
@@ -569,33 +592,33 @@ static int ReadMpqFileSectorFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos
     return ERROR_SUCCESS;
 }
 
-static int ReadMpqFilePatchFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpqFilePatchFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
 {
     TMPQPatcher Patcher;
     DWORD dwBytesToRead = dwToRead;
     DWORD dwBytesRead = 0;
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // Make sure that the patch file is loaded completely
-    if(nError == ERROR_SUCCESS && hf->pbFileData == NULL)
+    if(dwErrCode == ERROR_SUCCESS && hf->pbFileData == NULL)
     {
         // Initialize patching process and allocate data
-        nError = Patch_InitPatcher(&Patcher, hf);
-        if(nError != ERROR_SUCCESS)
-            return nError;
+        dwErrCode = Patch_InitPatcher(&Patcher, hf);
+        if(dwErrCode != ERROR_SUCCESS)
+            return dwErrCode;
 
         // Set the current data size
         Patcher.cbFileData = hf->pFileEntry->dwFileSize;
 
         // Initialize the patcher object with initial file data
         if(hf->pFileEntry->dwFlags & MPQ_FILE_SINGLE_UNIT)
-            nError = ReadMpqFileSingleUnit(hf, Patcher.pbFileData1, 0, Patcher.cbFileData, &dwBytesRead);
+            dwErrCode = ReadMpqFileSingleUnit(hf, Patcher.pbFileData1, 0, Patcher.cbFileData, &dwBytesRead);
         else
-            nError = ReadMpqFileSectorFile(hf, Patcher.pbFileData1, 0, Patcher.cbFileData, &dwBytesRead);
+            dwErrCode = ReadMpqFileSectorFile(hf, Patcher.pbFileData1, 0, Patcher.cbFileData, &dwBytesRead);
 
         // Perform the patching process
-        if(nError == ERROR_SUCCESS)
-            nError = Patch_Process(&Patcher, hf);
+        if(dwErrCode == ERROR_SUCCESS)
+            dwErrCode = Patch_Process(&Patcher, hf);
 
         // Finalize the patcher structure
         Patch_Finalize(&Patcher);
@@ -603,7 +626,7 @@ static int ReadMpqFilePatchFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos,
     }
 
     // If there is something to read, do it
-    if(nError == ERROR_SUCCESS)
+    if(dwErrCode == ERROR_SUCCESS)
     {
         if(dwFilePos < hf->cbFileData)
         {
@@ -617,21 +640,21 @@ static int ReadMpqFilePatchFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos,
         }
 
         // Set the proper error code
-        nError = (dwBytesRead == dwBytesToRead) ? ERROR_SUCCESS : ERROR_HANDLE_EOF;
+        dwErrCode = (dwBytesRead == dwBytesToRead) ? ERROR_SUCCESS : ERROR_HANDLE_EOF;
     }
 
     // Give the result to the caller
     if(pdwBytesRead != NULL)
         *pdwBytesRead = dwBytesRead;
-    return nError;
+    return dwErrCode;
 }
 
-static int ReadMpqFileLocalFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
+static DWORD ReadMpqFileLocalFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos, DWORD dwToRead, LPDWORD pdwBytesRead)
 {
     ULONGLONG FilePosition1 = dwFilePos;
     ULONGLONG FilePosition2;
     DWORD dwBytesRead = 0;
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     assert(hf->pStream != NULL);
 
@@ -643,7 +666,7 @@ static int ReadMpqFileLocalFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos,
     if(!FileStream_Read(hf->pStream, &FilePosition1, pvBuffer, dwToRead))
     {
         // If not all bytes have been read, then return the number of bytes read
-        if((nError = GetLastError()) == ERROR_HANDLE_EOF)
+        if((dwErrCode = SErrGetLastError()) == ERROR_HANDLE_EOF)
         {
             FileStream_GetPos(hf->pStream, &FilePosition2);
             dwBytesRead = (DWORD)(FilePosition2 - FilePosition1);
@@ -655,7 +678,7 @@ static int ReadMpqFileLocalFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos,
     }
 
     *pdwBytesRead = dwBytesRead;
-    return nError;
+    return dwErrCode;
 }
 
 //-----------------------------------------------------------------------------
@@ -663,9 +686,10 @@ static int ReadMpqFileLocalFile(TMPQFile * hf, void * pvBuffer, DWORD dwFilePos,
 
 bool WINAPI SFileReadFile(HANDLE hFile, void * pvBuffer, DWORD dwToRead, LPDWORD pdwRead, LPOVERLAPPED lpOverlapped)
 {
-    TMPQFile * hf = (TMPQFile *)hFile;
+    TFileEntry * pFileEntry;
+    TMPQFile * hf;
     DWORD dwBytesRead = 0;                      // Number of bytes read
-    int nError = ERROR_SUCCESS;
+    DWORD dwErrCode = ERROR_SUCCESS;
 
     // Always zero the result
     if(pdwRead != NULL)
@@ -673,60 +697,61 @@ bool WINAPI SFileReadFile(HANDLE hFile, void * pvBuffer, DWORD dwToRead, LPDWORD
     lpOverlapped = lpOverlapped;
 
     // Check valid parameters
-    if(!IsValidFileHandle(hFile))
+    if((hf = IsValidFileHandle(hFile)) == NULL)
     {
-        SetLastError(ERROR_INVALID_HANDLE);
+        SErrSetLastError(ERROR_INVALID_HANDLE);
         return false;
     }
 
     if(pvBuffer == NULL)
     {
-        SetLastError(ERROR_INVALID_PARAMETER);
+        SErrSetLastError(ERROR_INVALID_PARAMETER);
         return false;
     }
 
     // If we didn't load the patch info yet, do it now
     if(hf->pFileEntry != NULL && (hf->pFileEntry->dwFlags & MPQ_FILE_PATCH_FILE) && hf->pPatchInfo == NULL)
     {
-        nError = AllocatePatchInfo(hf, true);
-        if(nError != ERROR_SUCCESS || hf->pPatchInfo == NULL)
+        dwErrCode = AllocatePatchInfo(hf, true);
+        if(dwErrCode != ERROR_SUCCESS || hf->pPatchInfo == NULL)
         {
-            SetLastError(nError);
+            SErrSetLastError(dwErrCode);
             return false;
         }
     }
 
     // Clear the last used compression
+    pFileEntry = hf->pFileEntry;
     hf->dwCompression0 = 0;
 
     // If the file is local file, read the data directly from the stream
     if(hf->pStream != NULL)
     {
-        nError = ReadMpqFileLocalFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
+        dwErrCode = ReadMpqFileLocalFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
     }
 
     // If the file is a patch file, we have to read it special way
-    else if(hf->hfPatch != NULL && (hf->pFileEntry->dwFlags & MPQ_FILE_PATCH_FILE) == 0)
+    else if(hf->hfPatch != NULL && (pFileEntry->dwFlags & MPQ_FILE_PATCH_FILE) == 0)
     {
-        nError = ReadMpqFilePatchFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
+        dwErrCode = ReadMpqFilePatchFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
     }
 
     // If the archive is a MPK archive, we need special way to read the file
     else if(hf->ha->dwSubType == MPQ_SUBTYPE_MPK)
     {
-        nError = ReadMpkFileSingleUnit(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
+        dwErrCode = ReadMpkFileSingleUnit(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
     }
 
-    // If the file is single unit file, redirect it to read file 
-    else if(hf->pFileEntry->dwFlags & MPQ_FILE_SINGLE_UNIT)
+    // If the file is single unit file, redirect it to read file
+    else if(pFileEntry->dwFlags & MPQ_FILE_SINGLE_UNIT)
     {
-        nError = ReadMpqFileSingleUnit(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
+        dwErrCode = ReadMpqFileSingleUnit(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
     }
 
     // Otherwise read it as sector based MPQ file
     else
     {
-        nError = ReadMpqFileSectorFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
+        dwErrCode = ReadMpqFileSectorFile(hf, pvBuffer, hf->dwFilePos, dwToRead, &dwBytesRead);
     }
 
     // Increment the file position
@@ -738,13 +763,13 @@ bool WINAPI SFileReadFile(HANDLE hFile, void * pvBuffer, DWORD dwToRead, LPDWORD
 
     // If the read operation succeeded, but not full number of bytes was read,
     // set the last error to ERROR_HANDLE_EOF
-    if(nError == ERROR_SUCCESS && (dwBytesRead < dwToRead))
-        nError = ERROR_HANDLE_EOF;
+    if(dwErrCode == ERROR_SUCCESS && (dwBytesRead < dwToRead))
+        dwErrCode = ERROR_HANDLE_EOF;
 
     // If something failed, set the last error value
-    if(nError != ERROR_SUCCESS)
-        SetLastError(nError);
-    return (nError == ERROR_SUCCESS);
+    if(dwErrCode != ERROR_SUCCESS)
+        SErrSetLastError(dwErrCode);
+    return (dwErrCode == ERROR_SUCCESS);
 }
 
 //-----------------------------------------------------------------------------
@@ -793,7 +818,7 @@ DWORD WINAPI SFileGetFileSize(HANDLE hFile, LPDWORD pdwFileSizeHigh)
         return (DWORD)FileSize;
     }
 
-    SetLastError(ERROR_INVALID_HANDLE);
+    SErrSetLastError(ERROR_INVALID_HANDLE);
     return SFILE_INVALID_SIZE;
 }
 
@@ -808,7 +833,7 @@ DWORD WINAPI SFileSetFilePointer(HANDLE hFile, LONG lFilePos, LONG * plFilePosHi
     // If the hFile is not a valid file handle, return an error.
     if(!IsValidFileHandle(hFile))
     {
-        SetLastError(ERROR_INVALID_HANDLE);
+        SErrSetLastError(ERROR_INVALID_HANDLE);
         return SFILE_INVALID_POS;
     }
 
@@ -837,7 +862,7 @@ DWORD WINAPI SFileSetFilePointer(HANDLE hFile, LONG lFilePos, LONG * plFilePosHi
             break;
 
         case FILE_CURRENT:
-            
+
             // Retrieve the current file position
             if(hf->pStream != NULL)
             {
@@ -856,7 +881,7 @@ DWORD WINAPI SFileSetFilePointer(HANDLE hFile, LONG lFilePos, LONG * plFilePosHi
             break;
 
         default:
-            SetLastError(ERROR_INVALID_PARAMETER);
+            SErrSetLastError(ERROR_INVALID_PARAMETER);
             return SFILE_INVALID_POS;
     }
 
@@ -868,7 +893,7 @@ DWORD WINAPI SFileSetFilePointer(HANDLE hFile, LONG lFilePos, LONG * plFilePosHi
     {
         if(NewPosition > FileSize) // Position is negative
         {
-            SetLastError(ERROR_NEGATIVE_SEEK);
+            SErrSetLastError(ERROR_NEGATIVE_SEEK);
             return SFILE_INVALID_POS;
         }
     }
@@ -886,10 +911,9 @@ DWORD WINAPI SFileSetFilePointer(HANDLE hFile, LONG lFilePos, LONG * plFilePosHi
         if(!FileStream_Read(hf->pStream, &NewPosition, NULL, 0))
             return SFILE_INVALID_POS;
     }
-    else
-    {
-        hf->dwFilePos = (DWORD)NewPosition;
-    }
+
+    // Also, store the new file position to the TMPQFile struct
+    hf->dwFilePos = (DWORD)NewPosition;
 
     // Return the new file position
     if(plFilePosHigh != NULL)
